/**********************************************************************
Copyright (c) 2013 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
/**********************************************************************
Copyright (c) 2012 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
   ...
**********************************************************************/
package org.datanucleus.api.jpa;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.AttributeNode;
import javax.persistence.EntityGraph;
import javax.persistence.Subgraph;
import javax.persistence.metamodel.Attribute;

/**
 * Implementation of JPA2.1 EntityGraph.
 */
public class JPAEntityGraph<T> implements EntityGraph<T>
{
    String name;

    Class<T> type;

    List<AttributeNode<?>> attributeNodes;

    boolean mutable = true;

    public JPAEntityGraph(String name, Class type)
    {
        this.name = name;
        this.type = type;
    }

    public JPAEntityGraph<T> cloneMutableEntityGraph()
    {
        JPAEntityGraph<T> eg = new JPAEntityGraph<T>(name, type);
        if (attributeNodes != null)
        {
            eg.attributeNodes = new ArrayList();
            eg.attributeNodes.addAll(attributeNodes);
        }
        return eg;
    }

    public void setNotMutable()
    {
        this.mutable = false;
    }

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        this.name = name;
    }

    public Class<T> getType()
    {
        return type;
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addAttributeNodes(java.lang.String[])
     */
    public void addAttributeNodes(String... attributeNames)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        if (attributeNodes == null)
        {
            attributeNodes = new ArrayList<AttributeNode<?>>();
        }
        for (int i=0;i<attributeNames.length;i++)
        {
            AttributeNode node = new JPAAttributeNode<T>(attributeNames[i]);
            attributeNodes.add(node);
        }
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addAttributeNodes(javax.persistence.metamodel.Attribute<T,?>[])
     */
    public void addAttributeNodes(Attribute<T, ?>... attributes)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        if (attributeNodes == null)
        {
            attributeNodes = new ArrayList<AttributeNode<?>>();
        }
        for (int i=0;i<attributes.length;i++)
        {
            AttributeNode node = new JPAAttributeNode<T>(attributes[i].getName());
            attributeNodes.add(node);
        }
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#getAttributeNodes()
     */
    public List<AttributeNode<?>> getAttributeNodes()
    {
        return attributeNodes;
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addSubgraph(javax.persistence.metamodel.Attribute)
     */
    public <X> Subgraph<X> addSubgraph(Attribute<T, X> attribute)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addSubgraph(javax.persistence.metamodel.Attribute, java.lang.Class)
     */
    public <X> Subgraph<? extends X> addSubgraph(Attribute<T, X> attribute, Class<? extends X> type)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addSubgraph(java.lang.String)
     */
    public <X> Subgraph<X> addSubgraph(String attributeName)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addSubgraph(java.lang.String, java.lang.Class)
     */
    public <X> Subgraph<X> addSubgraph(String attributeName, Class<X> type)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addKeySubgraph(javax.persistence.metamodel.Attribute)
     */
    public <X> Subgraph<X> addKeySubgraph(Attribute<T, X> attribute)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addKeySubgraph(javax.persistence.metamodel.Attribute, java.lang.Class)
     */
    public <X> Subgraph<? extends X> addKeySubgraph(Attribute<T, X> attribute, Class<? extends X> type)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addKeySubgraph(java.lang.String)
     */
    public <X> Subgraph<X> addKeySubgraph(String attributeName)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addKeySubgraph(java.lang.String, java.lang.Class)
     */
    public <X> Subgraph<X> addKeySubgraph(String attributeName, Class<X> type)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    /* (non-Javadoc)
     * @see javax.persistence.EntityGraph#addSubclassSubgraph(java.lang.Class)
     */
    @SuppressWarnings("hiding")
    public <T> Subgraph<? extends T> addSubclassSubgraph(Class<? extends T> type)
    {
        if (!mutable)
        {
            throw new IllegalStateException("This EntityGraph is not mutable");
        }
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("Not yet implemented");
    }

    public String toString()
    {
        StringBuffer str = new StringBuffer();
        if (attributeNodes != null)
        {
            for (AttributeNode attr : attributeNodes)
            {
                if (str.length() > 0)
                {
                    str.append(",");
                }
                str.append(attr);
            }
        }

        return "EntityGraph [" + name + "] type=" + getClass().getName() + " attributes=[" + str.toString() + "]";
    }
}